﻿using Components.Aphid.Parser;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Components.Aphid.Interpreter
{
    public static class OperatorHelper
    {
        public static AphidObject Add(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("addition");
            }
            else if (x.Value is decimal && y.Value is decimal)
            {
                return new AphidObject((decimal)x.Value + (decimal)y.Value);
            }
            else if ((x.Value is sbyte ||
                x.Value is short ||
                x.Value is int ||
                x.Value is long ||
                x.Value is byte ||
                x.Value is ushort ||
                x.Value is uint ||
                x.Value is ulong ||
                x.Value is float ||
                x.Value is double) && 
                (y.Value is sbyte ||
                y.Value is short ||
                y.Value is int ||
                y.Value is long ||
                y.Value is byte ||
                y.Value is ushort ||
                y.Value is uint ||
                y.Value is ulong ||
                y.Value is float ||
                y.Value is double))
            {
                return new AphidObject(Convert.ToDecimal(x.Value) + Convert.ToDecimal(y.Value));
            }            
            else
            {
                return new AphidObject(Convert.ToString(x.Value) + Convert.ToString(y.Value));
            }
        }

        public static AphidObject Mod(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("modulo");
            }
            else
            {
                return new AphidObject((decimal)x.Value % (decimal)y.Value);
            }
        }

        public static AphidObject BinaryOr(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("binary or");
            }
            else
            {
                return new AphidObject((decimal)((long)(decimal)x.Value | (long)(decimal)y.Value));
            }
        }

        public static AphidObject BinaryAnd(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("binary and");
            }
            else
            {
                return new AphidObject((decimal)((long)(decimal)x.Value & (long)(decimal)y.Value));
            }
        }

        public static AphidObject BinaryShiftLeft(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("binary shift left");
            }
            else
            {
                return new AphidObject((decimal)((int)(decimal)x.Value << (int)(decimal)y.Value));
            }
        }

        public static AphidObject BinaryShiftRight(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("binary shift right");
            }
            else
            {
                return new AphidObject((decimal)((int)(decimal)x.Value << (int)(decimal)y.Value));
            }
        }

        public static AphidObject Xor(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("exclusive or");
            }
            else
            {
                return new AphidObject((decimal)((int)(decimal)x.Value ^ (int)(decimal)y.Value));
            }
        }

        public static AphidObject Subtract(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("subtraction");
            }
            else
            {
                return new AphidObject(((decimal)x.Value) - (decimal)y.Value);
            }
        }

        public static AphidObject Multiply(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("multiplication");
            }
            else
            {
                object val;
                if (x.Value is Decimal)
                {
                    val = ((decimal)x.Value) * (decimal)y.Value;
                }
                else if (x.Value is string)
                {
                    var s = x.Value as string;
                    var sb = new StringBuilder();

                    for (int i = 0; i < (decimal)y.Value; i++)
                    {
                        sb.Append(s);
                    }

                    return new AphidObject(sb.ToString());

                    //val = sb.ToString();
                }
                else
                {
                    throw new AphidRuntimeException("Could not multiply type");
                }

                return new AphidObject(val);
            }
        }

        public static AphidObject Divide(AphidObject x, AphidObject y)
        {
            if (x == null || y == null)
            {
                throw new AphidOperationException("division");
            }
            else
            {
                return new AphidObject(((decimal)x.Value) / (decimal)y.Value);
            }
        }
    }
}
